##
#
# OpenERP Sink for Conduit Sync Application
# Copyright (C) 2010-TODAY Mantavya Gajjar <http://www.mantavyagajjar.in>.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##

import xmlrpclib
import vobject

import conduit
from conduit.datatypes import File
from conduit.datatypes import DataType
from conduit import TypeConverter
from conduit.dataproviders import DataProvider

import conduit.datatypes.Contact as Contact

from gettext import gettext as _

import logging
log = logging.getLogger("modules.OpenERP")

MODULES = {
    "OpenERPSink":{"type": "dataprovider"},
    "ContactConverter":{"type": "converter"}
}

class OpenERPSink(DataProvider.DataSink):
    """
    OpenERP Data Importing Module
    """

    _name_ = "OpenERP"
    _description_ = "OpenERP Sink, Allows to import data in OpenERP"
    _category_ = conduit.dataproviders.CATEGORY_OFFICE
    _module_type_ = "sink"
    _in_type_ = "contact"
    _out_type_ = "contact"
    _icon_ = "applications-internet"
    _configurable_ = True

    def __init__(self, *args):
        DataProvider.DataSink.__init__(self)
        self.server = False
        self.url = "http://%s:%s/xmlrpc/common"
        self.dbname = ""
        self.user = 0
        self.update_configuration(
            host = "localhost",
            port = "8069",
            database = "",
            username = "",
            password = "",
            model  = ""
        )
        
    def config_setup(self, config):
        config.add_section(_("OpenERP Server Information"))
        host_config = config.add_item(_("Host"), "text",
            config_name = 'host',
        )
        port_config = config.add_item(_("Port"), "text",
            config_name = 'port',
        )
        
        def _load_db(button):
            
            config.apply_config(items = [host_config, port_config])
            
            url = "http://%s:%s/xmlrpc/db" % (self.host, self.port)
            sock = xmlrpclib.ServerProxy(url)
            dblist = sock.list()
            
            if dblist:
                self.dbname = dblist[0][0]
                config.add_section(_("Connection Information"))
                dblist_config = config.add_item(_("DB Name"), "combo",
                    config_name = 'database',
                    choices = dblist,
                    initial_value_callback = lambda *a:self.dbname
                )
                username = config.add_item(_("Username"), "text",
                    config_name = 'username',
                )
                password = config.add_item(_("Password"), "text",
                    config_name = 'password',
                )
                def _login(button):
                    config.apply_config(items = [host_config, port_config, dblist_config, username, password])
                    
                    url = "http://%s:%s/xmlrpc/common" % (self.host, self.port)
                    sock = xmlrpclib.ServerProxy(url)
                    self.user = sock.login(self.database, self.username, self.password)
                    if self.user:
                        url = "http://%s:%s/xmlrpc/object" % (self.host, self.port)
                        self.server = xmlrpclib.ServerProxy(url)
                        
                config.add_item("Login", "button", image="dialog-password", action = _login)

        config.add_item("Connect", "button", image="dialog-password", action = _load_db)
        
    def refresh(self):
        #DataProvider.DataSource.refresh(self)
        pass
    
    def get_UID(self):
        return "OpenERP://%s@%s" % (self.username, self.database)
        
    def is_configured (self, isSource, isTwoWay):
        if self.user > 0:
            return True
        return False
    
    def put(self, contact, overwrite, LUID=None):
        DataProvider.DataSink.put(self, contact, overwrite, LUID)
        
        v = vobject.readOne(contact.get_vcard_string())

        partner = {
            'name':v.fn.value,
            'ref':v.fn.value[0:3].upper(),
            'user_id':self.user
        }
        partner_id = self.server.execute(self.database, self.user, self.password, 'res.partner', 'create', partner)
        address_id = partner_id
        if v.contents.get('adr', False):
            address = {
                'partner_id':partner_id,
                'name':v.fn.value,
                'street':v.adr.value.street,
                'street2':v.adr.value.extended,
                'city':v.adr.value.city,
                'zip':v.adr.value.code,
                'user_id':self.user
            }
            if v.contents.get('tel',False):
                address.update({'mobile':v.contents.get('tel')[0].value})
            if v.contents.get('tel',False) and len(v.contents.get('tel')) > 1:
                address.update({'phone':v.contents.get('tel')[1].value})
            address_id = self.server.execute(self.database, self.user, self.password, 'res.partner.address', 'create', address)
        return contact
    
class TestDataType(DataType.DataType):
    _name_ = "oocontacts"
    
    def __init__(self):
        DataType.DataType.__init__(self)
        
    def __str__(self):
        return "TestDataType"

class ContactConverter(TypeConverter.Converter):
    def __init__(self):
        self.conversions =  {
            "contact,oocontacts":self._to_contacts,
        }
    def _to_contacts(self, text, **kwargs):
        pass
        
